#include <config.h>

#include "antilink.h"
#include "safeio.h"
#include "crypto.h"
#include "mime.h"
#include <glib.h>
#include "scgi.h"

#ifdef WITH_DMALLOC
# include <dmalloc.h>
#endif

static char *readkey(void)
{
    int fd;
    static char key[ANTILINK_KEY_LEN + 1U];
    
    if ((fd = open(ANTILINK_KEY_FILE, O_RDONLY)) == -1) {
        return NULL;
    }
    if (safe_read(fd, key, sizeof key - 1U) != sizeof key - 1U) {
        close(fd);
        return NULL;
    }
    close(fd);
    
    return key;
}

static void spawncgi(const char * const base, const char * const file)
{
    int fd;
    ssize_t readen;
    char path[MAXPATHLEN];

    if (*file == '/' || *file == '.' || strstr(file, "/.") != NULL) {
        errno = EPERM;
        return;
    }
    if (SNCHECK(snprintf(path, sizeof path, "%s/%s", base, file),
                sizeof path)) {
        errno = E2BIG;
        return;
    }
    execl(path, path, NULL);
}

int main(void)
{
    CGIVars *cgi_vars;
    const char *file;
    const char *remote_addr;
    const char *readen_key;
    const char *private_key;
    char *tohash;
    size_t sizeof_tohash;
    const char *hashed;
    char salt[9];    

    salt[sizeof salt - 1U] = 0;
    if ((private_key = readkey()) == NULL) {
        return 1;
    }
    sizeof_tohash = (size_t) 8U + 42U + ANTILINK_KEY_LEN + (size_t) 1U;
    if ((tohash = ALLOCA(sizeof_tohash)) == NULL) {
        return 2;
    }
    if ((cgi_vars = cgi_init(42000, 420, 42, 420, 42)) == NULL ||
        (file = cgi_get(cgi_vars, ANTILINK_LINK_VAR_FILE)) == NULL ||
        (readen_key = cgi_get(cgi_vars, ANTILINK_LINK_VAR_KEY)) == NULL ||
        strlen(readen_key) < (size_t) 9U ||
        (remote_addr = getenv("REMOTE_ADDR")) == NULL) {
        return 3;
    }
    memcpy(salt, readen_key, (size_t) 8U);
    snprintf(tohash, sizeof_tohash, "%s%s%s",
             salt, remote_addr, private_key);
    if ((hashed = crypto_hash_sha1(tohash, 1)) == NULL) {
        return 4;
    }
    if (strcmp(hashed, readen_key + 8) != 0) {
        puts(ANTILINK_ERROR_PREFIX "Sorry - bad crypto key");
        return 5;
    }
    spawncgi(ANTILINK_PATH_HTML_CHECKKEY_CGI, file);
#ifndef HIDE_ERRORS
    printf(ANTILINK_ERROR_PREFIX "%s", strerror(errno));
#endif
    cgi_destroy(cgi_vars);
    ALLOCA_FREE(tohash);
    
    return 0;
}
