#include <config.h>

#include "antilink.h"
#include "safeio.h"
#ifdef WITH_DMALLOC
# include <dmalloc.h>
#endif

ssize_t safe_read(const int fd, void * const buf_, size_t maxlen)
{
    unsigned char *buf = (unsigned char *) buf_;
    ssize_t readen;
    
    do {
        while ((readen = read(fd, buf, maxlen)) < (ssize_t) 0 && 
               errno == EINTR);
        if (readen < (ssize_t) 0 || readen > (ssize_t) maxlen) {
            return readen;
        }
        if (readen == (ssize_t) 0) {
            ret:
            return (ssize_t) (buf - (unsigned char *) buf_);
        }
        maxlen -= readen;
        buf += readen;
    } while (maxlen > (ssize_t) 0);
    goto ret;
}

int safe_write(const int fd, const void *buf_, size_t count)
{
    register const char *buf = (const char *) buf_;
    ssize_t written;
        
    while (count > (size_t) 0) {
        for (;;) {
            if ((written = write(fd, buf, count)) <= (ssize_t) 0) {
                if (errno == EAGAIN) {
                    sleep(1);
                } else if (errno != EINTR) {
                    return -1;
                }
                continue;
            }
            break;
        }
        buf += written;
        count -= written;
    }
    return 0;
}
